from typing import Callable, Dict, List, Tuple
import anltk_pybind

from anltk_pybind import AR2BW, BW2AR, AR2SBW, SBW2AR



class TafqitOptions:

    def __init__(self,
                 is_feminine: bool = False,
                 use_comma: bool = False,
                 split_hundred: bool = False,
                 use_miah: bool = False,
                 use_billion: bool = False,
                 has_followup_text: bool = False,
                 is_accusative: bool = False,
                 subjects: List[str] = [],
                 use_legal_form: bool = False) -> None:

        self.is_feminine = is_feminine
        self.use_comma = use_comma
        self.split_hundred = split_hundred
        self.use_miah = use_miah
        self.use_billion = use_billion
        self.has_followup_text = has_followup_text
        self.is_accusative = is_accusative
        self.subjects = subjects
        self.use_legal_form = use_legal_form


def tafqit(num: int, opts: TafqitOptions = TafqitOptions()) -> str:
    """Converts a number into Arabic spoken form
    * {is_feminine}     : "on": Generate string for a Feminine subject (أرقام بصيغة المؤنث).
    *                        The default is the Masculine form.
    * {use_miah}         : "on": Use Mi'ah for Hundreds (مئة بدل مائة). Default is Ma'ah "مائة".
    * {use_comma}        : "on": Insert comma between triplet words.
    * {split_hundred}    : "on": Split number from hundred words (فصل الرقم عن المئة).
                       i.e. ثلاث مائة. Default "No Split" i.e. (ثلاثمائة).
    * {use_billion}     : "on": Use Billions (بليون) instead of Miliard (مليار).
    * {has_followup_text} : "on": Indicates that there will be text to follow the resulting text.
    *                  This permits proper subject name to be added after the resulting text.
    * {is_accusative}           : "on": Text is produced in Accusative/Genitive (جر/نصب) case.
    *                  The default is the Nominative cse (رفع).
    *
    * {subjects}       : An optional array holding the Subject Name to be counted in 4 forms as follows:
    *                  [0] = Deafult Name Singular      (e.g. "كتاب/تفاحة/دينار").
    *                  [1] = Name for 2's (double)      (e.g. "كتابان/تفاحتان/ديناران").
    *                  [2] = Name for plural            (e.g. "كتب/تفاحات/دنانير").
    *                  [3] = Name Singular with Tanween (e.g. "كتابًا/تفاحةً/دينارًا").
    *                  The subject name will be added to the resulting string in acordance
    *                  with the number grammar rules.
    * {use_legal_form}        : "on" Uses the lagal form of output text.
    """
    opts_ = anltk_pybind.TafqitOptionsType()
    opts_.is_feminine = opts.is_feminine
    opts_.use_comma = opts.use_comma
    opts_.split_hundred = opts.split_hundred
    opts_.use_miah = opts.use_miah
    opts_.use_billion = opts.use_billion
    opts_.has_followup_text = opts.has_followup_text
    opts_.is_accusative = opts.is_accusative
    opts_.subjects = opts.subjects
    opts_.use_legal_form = opts.use_legal_form
    return anltk_pybind.tafqit(num, opts_)


def transliterate(text: str, mapping: anltk_pybind.CharMapping) -> str:
    """ Convert between famous characters mapping

    Args:
        text (str): input text
        mapping (anltk_pybind.CharMapping): Mapping type 
            AR2BW : Arabic to buckwalter
            AR2SBW : Arabic to safe buckwalter
            BW2AR: Buckwalter to arabic
            SBW2AR : Safe buckwalter to arabic
    Returns:
        [str]
    Example : 
        `anltk.transliterate("أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ", anltk.AR2BW) `\n 
        `'>bjd hwz HTy klmn sEfS qr$t vx* DZg ` 
    """
    return anltk_pybind.transliterate(text, mapping)


def tokenize_words(text: str) -> List[str]:
    return anltk_pybind.tokenize_words(text)


def is_tashkeel(c: str) -> bool:
    return anltk_pybind.is_tashkeel(c)


def is_arabic_alpha(c: str) -> bool:
    return anltk_pybind.is_arabic_alpha(c)


def is_small(c: str) -> bool:
    return anltk_pybind.is_small(c)


def is_indic_digit(c: str) -> bool:
    return anltk_pybind.is_indic_digit(c)


def is_valid_kalima(c: str) -> bool:
    return anltk_pybind.is_valid_kalima(c)


def is_shamsi(c: str) -> bool:
    return anltk_pybind.is_shamsi(c)


def is_qamari(c: str) -> bool:
    return anltk_pybind.is_qamari(c)


def remove_tashkeel(text: str) -> str:
    return anltk_pybind.remove_tashkeel(text)


def remove_small(text: str) -> str:
    return anltk_pybind.remove_small(text)


def remove_non_alpha(text: str, stop_list: str = ' ') -> str:
    return anltk_pybind.remove_non_alpha(text, stop_list)


def remove_non_alpha_and_tashkeel(text: str, stop_list: str = ' ') -> str:
    return anltk_pybind.remove_non_alpha_and_tashkeel(text, stop_list)


def remove_non_alphanumeric(text: str, stop_list: str = ' ') -> str:
    return anltk_pybind.remove_non_alphanumeric(text, stop_list)


def remove_non_alphanumeric_and_tashkeel(text: str,
                                         stop_list: str = ' ') -> str:
    return anltk_pybind.remove_non_alphanumeric_and_tashkeel(text, stop_list)


def remove_kasheeda(text: str) -> str:
    return anltk_pybind.remove_kasheeda(text)


def normalize_hamzat(text: str) -> str:
    return anltk_pybind.normalize_hamzat(text)


def duplicate_shadda_letter(text: str) -> str:
    return anltk_pybind.duplicate_shadda_letter(text)


def remove_if(text: str, stop_list: str, func: Callable) -> str:
    return anltk_pybind.remove_if(text, stop_list, func)


def replace(text: str, chars_map: Dict[str, str]):
    return anltk_pybind.replace(text, chars_map)


def replace_str(text: str, chars_map: Dict[str, str]):
    return anltk_pybind.replace_str(text, chars_map)


def replace_if(text: str, func: Callable, replacement: str) -> str:
    return anltk_pybind.replace_if(text, func, replacement)


def fold_if(text: str, func: Callable) -> str:
    return anltk_pybind.fold_if(text, func)


def fold_white_spaces(text: str) -> str:
    return anltk_pybind.fold_white_spaces(text)


def split(text: str,
          delimeters: str = ' ',
          keep_delimeters: bool = False) -> List[str]:
    return anltk_pybind.split(text, delimeters, keep_delimeters)


def normalize_to_heh(text: str) -> str:
    return anltk_pybind.normalize_to_heh(text)


def normalize_to_teh(text: str) -> str:
    return anltk_pybind.normalize_to_teh(text)


def tokenize_if(text: str, funcs: List[Callable]) -> List[Tuple[int, str]]:
    return anltk_pybind.tokenize_if(text, funcs)


class NoiseGenerator:

    def __init__(self) -> None:
        self.ng = anltk_pybind.NoiseGenerator() # Default random seed

    def set_seed(self, seed: int):
        self.ng.set_seed(seed)

    def swap_adjacent_chars(self, text: str, n: int = 1) -> str:
        return self.ng.swap_adjacent_chars(text, n)

    def insert_random_chars(self, text: str, n: int = 1) -> str:
        return self.ng.insert_random_chars(text, n)
    
    def remove_random_chars(self, text:str, n : int = 1) -> str:
        return self.ng.remove_random_chars(text, n)

    def replace_random_chars(self, text:str, n : int = 1) -> str:
        return self.ng.replace_random_chars(text, n)

    def join_random_words(self, text:str, n : int = 1) -> str:
        return self.ng.join_random_words(text, n)
    
    def swap_random_words(self, text:str, n : int = 1) -> str:
        return self.ng.swap_random_words(text, n) 


import os
import setuptools
import platform
import pybind11
from pybind11.setup_helpers import Pybind11Extension, build_ext

# Figure out environment for cross-compile
anltk_source = os.getenv("ANLTK_SOURCE",
                         os.path.abspath(os.path.dirname(__file__)))            
system = os.environ.get('ANLTK_PLATFORM', platform.system())

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()


def _get_project_version():
    version_path = os.path.join(anltk_source, "python", "anltk_module",
                                "anltk", "version.py")
    version = {}
    with open(version_path, encoding="utf-8") as fp:
        exec(fp.read(), version)
    return version["__version__"]


include_dirs = [
    pybind11.get_include(),
    os.path.join(anltk_source, 'anltk', 'include'),
    os.path.join(anltk_source, 'subprojects', 'utfcpp', 'source')
]

cflags = ['-O3']
ldflags = []

if system == 'Windows':
    cflags = ['/O2', '/utf-8']

anltk_src_files = map(str,
                      os.listdir(os.path.join(anltk_source, 'anltk', 'src')))
anltk_src_cpp = list(filter(lambda x: x.endswith('.cpp'), anltk_src_files))
anltk_src_cpp.remove('unicode_normalization.cpp')
anltk_src_cpp = list(
    map(lambda x: str(os.path.join(anltk_source, 'anltk', 'src', x)),
        anltk_src_cpp))

anltk_pybind_module_cpp = os.path.join(anltk_source, 'python', 'anltk_module',
                                       'anltk', 'pybind', 'anltk_pybind.cpp')
ext_modules = [
    Pybind11Extension("anltk_pybind",
                      sources=anltk_src_cpp + [anltk_pybind_module_cpp],
                      extra_compile_args=cflags,
                      cxx_std=17,
                      extra_link_args=ldflags,
                      include_dirs=include_dirs,
                      define_macros=[("UTF_CPP_CPLUSPLUS", "201703L"),
                                     ("BUILDING_PYBIND11", "1")])
]

setuptools.setup(
    name="anltk",
    version=_get_project_version(),
    author="Abdullah Alattar",
    author_email="abdullah.mohammad.alattar@gmail.com",
    description="Arabic language processing toolkit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Abdullah-AlAttar/anltk",
    cmdclass={"build_ext": build_ext},
    python_requires='>=3',
    project_urls={
        "Source": "https://github.com/Abdullah-AlAttar/anltk",
    },
    ext_modules=ext_modules,
    zip_safe=False,
    include_package_data=False,
    package_dir={str(''): str('python/anltk_module')},
    packages=[str('anltk'),
              str('anltk.constants'),
              str('anltk.tests')],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Artificial Intelligence',
        'Natural Language :: Arabic',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
    ],
    keywords=['NLP', 'Arabic', 'python', 'arabic', 'c++'])



import anltk

# def test_has_spaces():

#     ar_text = 'أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_empty():

#     ar_text = ''
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Has_double_consecutive_shadda():

#     ar_text = 'بسّّم'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_first_character_tashkeel():

#     ar_text = 'ًبسم'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_First_Character_WAW_HAMZA():

#     ar_text = 'ؤامر'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_First_Character_YEH_HAMZA():

#     ar_text = 'ئاسر'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Alef_Maqsura_not_last_Character():

#     ar_text = 'ىا ليلى'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Teh_Marbuta_not_last_Character():

#     ar_text = 'ىا لةيلى'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Three_consectuve_harakat():

#     ar_text = 'ىاًَُ لةيلى'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_double_Teh_Marbuta():

#     ar_text = 'ىا لةةيلى'
#     assert anltk.is_valid_kalima(ar_text) == False

# def test_Valid():

#     ar_text = 'فأسقيناكموه'
#     assert anltk.is_valid_kalima(ar_text) == True

# def test_Valid1():

#     ar_text = 'فأسًقيٍنََاّكمٍوُه'
#     assert anltk.is_valid_kalima(ar_text) == True


def test_tokenize():

    text = 'بسم الله@ الرحمن23 الرحيم.'

    assert anltk.tokenize_words(text) == [
        'بسم', 'الله', '@', 'الرحمن', '23', 'الرحيم', '.'
    ]





def test_transliterate():

    assert anltk.transliterate(
        'أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ',
        anltk.AR2BW) == '>bjd hwz HTy klmn sEfS qr$t vx* DZg'

    # Safe Buckwalter
    assert anltk.transliterate(
        'أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ',
        anltk.AR2SBW) == 'Objd hwz HTy klmn sEfS qrct vxV DZg'

    assert anltk.transliterate(
        '>bjd hwz HTy klmn sEfS qr$t vx* DZg',
        anltk.BW2AR) == 'أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ'

    assert anltk.transliterate(
        'Objd hwz HTy klmn sEfS qrct vxV DZg',
        anltk.SBW2AR) == 'أبجد هوز حطي كلمن سعفص قرشت ثخذ ضظغ'


def test_remove_tashkeel():
    assert anltk.remove_tashkeel('فرَاشَةٌ مُلَوَّنَةٌ') == 'فراشة ملونة'


def test_remove_non_alpha():
    assert anltk.remove_non_alpha('فرَاdشَةٌ مُلَوَّنَ3ةٌ!',
                                  stop_list='! ') == 'فراشة ملونة!'
    assert anltk.remove_non_alphanumeric('فرَاdشَةٌ مُلَوَّ3نَةٌ!',
                                         stop_list='! ') == 'فراشة ملو3نة!'

    assert anltk.remove_non_alphanumeric_and_tashkeel(
        'فرَاdشَةٌ مُلَوَّ3نَةٌ!', stop_list='! ') == 'فرَاشَةٌ مُلَوَّ3نَةٌ!'
    assert anltk.remove_kasheeda('بســــم الـــله') == 'بسم الله'


def test_remove_if():
    assert anltk.remove_if('باب الحارة', ' ', lambda x: x == 'ب') == 'ا الحارة'


def test_folding():
    assert anltk.fold_white_spaces(
        'بسم     الله    الرحمن ال   رحيم\t\t') == 'بسم الله الرحمن ال رحيم '


def test_splitting():
    assert anltk.split('ffsl بسم اللهْ') == ['ffsl', 'بسم', 'اللهْ']
    assert anltk.split('يا الله, يا رحمن يا رحيم.',
                       delimeters='.,',
                       keep_delimeters=False) == [
                           'يا الله', ' يا رحمن يا رحيم'
                       ]
    assert anltk.split('يا الله, يا رحمن يا رحيم.',
                       delimeters='.,',
                       keep_delimeters=True) == [
                           'يا الله,', ' يا رحمن يا رحيم.'
                       ]


def test_replace():
    assert anltk.replace_if('ffsl بسم اللهْ',
                            lambda x: not anltk.is_arabic_alpha(x),
                            '!') == '!!!!!بسم!الله!'


def test_normalize():
    assert anltk.normalize_to_heh(
        'الحديقة! الجديدةُ كانت مشرقة إشراقةً تهيم على الشفاه'
    ) == 'الحديقه! الجديدهُ كانت مشرقه إشراقهً تهيم على الشفاه'
    assert anltk.normalize_to_teh(
        'الحديقه! الجديدهُ كانت مشرقه إشراقهً تهيم على الشفاه'
    ) == 'الحديقة! الجديدةُ كانت مشرقة إشراقةً تهيم على الشفاة'


def test_tokenize_if():
    text = "إِنَّ الَّذِينَ كَفَرُوا سَوَاءٌ عَلَيْهِمْ أَأَنذَرْتَهُمْ أَمْ لَمْ تُنذِرْهُمْ لَا يُؤْمِنُونَ ."

    assert anltk.tokenize_if(text,
                             [anltk.is_tashkeel, anltk.is_arabic_alpha]) == [
                                 (1, 'إ'), (0, 'ِ'), (1, 'ن'), (0, 'َّ'),
                                 (-1, ' '), (1, 'ال'), (0, 'َّ'), (1, 'ذ'),
                                 (0, 'ِ'), (1, 'ين'), (0, 'َ'), (-1, ' '),
                                 (1, 'ك'), (0, 'َ'), (1, 'ف'), (0, 'َ'),
                                 (1, 'ر'), (0, 'ُ'), (1, 'وا'), (-1, ' '),
                                 (1, 'س'), (0, 'َ'), (1, 'و'), (0, 'َ'),
                                 (1, 'اء'), (0, 'ٌ'), (-1, ' '), (1, 'ع'),
                                 (0, 'َ'), (1, 'ل'), (0, 'َ'), (1, 'ي'),
                                 (0, 'ْ'), (1, 'ه'), (0, 'ِ'), (1, 'م'),
                                 (0, 'ْ'), (-1, ' '), (1, 'أ'), (0, 'َ'),
                                 (1, 'أ'), (0, 'َ'), (1, 'نذ'), (0, 'َ'),
                                 (1, 'ر'), (0, 'ْ'), (1, 'ت'), (0, 'َ'),
                                 (1, 'ه'), (0, 'ُ'), (1, 'م'), (0, 'ْ'),
                                 (-1, ' '), (1, 'أ'), (0, 'َ'), (1, 'م'),
                                 (0, 'ْ'), (-1, ' '), (1, 'ل'), (0, 'َ'),
                                 (1, 'م'), (0, 'ْ'), (-1, ' '), (1, 'ت'),
                                 (0, 'ُ'), (1, 'نذ'), (0, 'ِ'), (1, 'ر'),
                                 (0, 'ْ'), (1, 'ه'), (0, 'ُ'), (1, 'م'),
                                 (0, 'ْ'), (-1, ' '), (1, 'ل'), (0, 'َ'),
                                 (1, 'ا'), (-1, ' '), (1, 'ي'), (0, 'ُ'),
                                 (1, 'ؤ'), (0, 'ْ'), (1, 'م'), (0, 'ِ'),
                                 (1, 'ن'), (0, 'ُ'), (1, 'ون'), (0, 'َ'),
                                 (-1, ' .')
                             ]
def test_tafqit():
    opts = anltk.TafqitOptions()
    assert anltk.tafqit(2000, opts) == "ألفان"
    assert anltk.tafqit(2020, opts) == "ألفان وعشرون"
    assert anltk.tafqit(15000120, opts) == "خمسة عشر مليونًا ومائة وعشرون"
    assert anltk.tafqit(0, opts) == "صفر"
    assert anltk.tafqit(1, opts) == "واحد"
    assert anltk.tafqit(2, opts) == "اثنان"
    assert anltk.tafqit(3, opts) == "ثلاثة"
    assert anltk.tafqit(4, opts) == "أربعة"
    assert anltk.tafqit(5, opts) == "خمسة"
    assert anltk.tafqit(6, opts) == "ستة"
    assert anltk.tafqit(7, opts) == "سبعة"
    assert anltk.tafqit(8, opts) == "ثمانية"
    assert anltk.tafqit(9, opts) == "تسعة"  
    assert anltk.tafqit(10, opts) == "عشرة"
    assert anltk.tafqit(22, opts) == "اثنان وعشرون"
    assert anltk.tafqit(30, opts) == "ثلاثون"   
    assert anltk.tafqit(119, opts) == "مائة وتسعة عشر"
    assert anltk.tafqit(120, opts) == "مائة وعشرون"
    assert anltk.tafqit(199, opts) == "مائة وتسعة وتسعون"   
    assert anltk.tafqit(200, opts) == "مائتان"  
    assert anltk.tafqit(300, opts) == "ثلاثمائة"
    assert anltk.tafqit(311, opts) == "ثلاثمائة وأحد عشر"   
    assert anltk.tafqit(1019, opts) == "ألف وتسعة عشر"
    assert anltk.tafqit(3019, opts) == "ثلاثة آلاف وتسعة عشر"
    assert anltk.tafqit(10000, opts) == "عشرة آلاف"
    assert anltk.tafqit(10500, opts) == "عشرة آلاف وخمسمائة"
    assert anltk.tafqit(100500, opts) == "مائة ألف وخمسمائة"
    assert anltk.tafqit(110500, opts) == "مائة وعشرة آلاف وخمسمائة"
    assert anltk.tafqit(-110500, opts) == "سالب مائة وعشرة آلاف وخمسمائة"
    assert anltk.tafqit(99999, opts) == "تسعة وتسعون ألفًا وتسعمائة وتسعة وتسعون"
    assert anltk.tafqit(999999, opts) == "تسعمائة وتسعة وتسعون ألفًا وتسعمائة وتسعة وتسعون"
    assert anltk.tafqit(1000000000, opts) == "مليار"
    assert anltk.tafqit(1000000001, opts) == "مليار وواحد"
    assert anltk.tafqit(10000000000, opts) == "عشرة مليارات"
    assert anltk.tafqit(1000000000000, opts) == "ترليون"
    
    opts.is_feminine = True
    assert anltk.tafqit(2000, opts) == "ألفان"
    assert anltk.tafqit(2020, opts) == "ألفان وعشرون"
    assert anltk.tafqit(15000120, opts) == "خمسة عشر مليونًا ومائة وعشرون"
    assert anltk.tafqit(0, opts) == "صفر"
    assert anltk.tafqit(1, opts) == "واحدة"
    assert anltk.tafqit(2, opts) == "اثنتان"
    assert anltk.tafqit(3, opts) == "ثلاث"
    assert anltk.tafqit(4, opts) == "أربع"
    assert anltk.tafqit(5, opts) == "خمس"
    assert anltk.tafqit(6, opts) == "ست"
    assert anltk.tafqit(7, opts) == "سبع"
    assert anltk.tafqit(8, opts) == "ثمان"
    assert anltk.tafqit(9, opts) == "تسع"   
    assert anltk.tafqit(10, opts) == "عشر"
    assert anltk.tafqit(22, opts) == "اثنتان وعشرون"
    assert anltk.tafqit(30, opts) == "ثلاثون"   
    assert anltk.tafqit(119, opts) == "مائة وتسع عشرة"
    assert anltk.tafqit(120, opts) == "مائة وعشرون"
    assert anltk.tafqit(199, opts) == "مائة وتسع وتسعون"    
    assert anltk.tafqit(200, opts) == "مائتان"  
    assert anltk.tafqit(300, opts) == "ثلاثمائة"
    assert anltk.tafqit(311, opts) == "ثلاثمائة وإحدى عشرة" 
    assert anltk.tafqit(1019, opts) == "ألف وتسع عشرة"
    assert anltk.tafqit(3019, opts) == "ثلاثة آلاف وتسع عشرة"
    assert anltk.tafqit(10000, opts) == "عشرة آلاف"
    assert anltk.tafqit(10500, opts) == "عشرة آلاف وخمسمائة"
    assert anltk.tafqit(100500, opts) == "مائة ألف وخمسمائة"
    assert anltk.tafqit(110500, opts) == "مائة وعشرة آلاف وخمسمائة"
    assert anltk.tafqit(-110500, opts) == "سالب مائة وعشرة آلاف وخمسمائة"
    assert anltk.tafqit(99999, opts) == "تسعة وتسعون ألفًا وتسعمائة وتسع وتسعون"
    assert anltk.tafqit(999999, opts) == "تسعمائة وتسعة وتسعون ألفًا وتسعمائة وتسع وتسعون"
    assert anltk.tafqit(1000000000, opts) == "مليار"
    assert anltk.tafqit(1000000001, opts) == "مليار وواحدة"
    assert anltk.tafqit(10000000000, opts) == "عشرة مليارات"
    assert anltk.tafqit(1000000000000, opts) == "ترليون"
    
    
def test_noise():
    gen = anltk.NoiseGenerator()
    gen.set_seed(3)
    
    assert gen.insert_random_chars('صوت صفير البلبل') == 'صوت صشفير البلبل'
    assert gen.swap_adjacent_chars('صوت صفير البلبل') == 'صوت فصير البلبل'
    assert gen.remove_random_chars('صوت صفير البلبل') == 'صوت صفير البلل'
    assert gen.replace_random_chars('صوت صفير البلبل') == 'صوت صفير ءلبلبل'
    assert gen.join_random_words('صوت صفير البلبل') == 'صوت صفيرالبلبل'
    assert gen.swap_random_words('صوت صفير البلبل') == 'صفير صوت البلبل'